@@ -5,10 +5,25 @@ use strict;
 use warnings;
 use Carp qw/croak/;
 
-our $VERSION = '1.00';
+our $VERSION = '1.04_01';
 
 require XSLoader;
 XSLoader::load('Class::XSAccessor', $VERSION);
+require Class::XSAccessor::Heavy;
+
+sub _make_hash {
+  my $ref = shift;
+
+  if (ref ($ref)) {
+    if (ref($ref) eq 'ARRAY') {
+      $ref = { map { $_ => $_ } @$ref }
+    } 
+  } else {
+    $ref = { $ref, $ref };
+  }
+
+  return $ref;
+}
 
 sub import {
   my $own_class = shift;
@@ -18,74 +33,53 @@ sub import {
 
   $caller_pkg = $opts{class} if defined $opts{class};
 
-  my $replace = $opts{replace} || 0;
-  my $chained = $opts{chained} || 0;
-
-  my $read_subs      = $opts{getters} || {};
-  my $set_subs       = $opts{setters} || {};
-  my $acc_subs       = $opts{accessors} || {};
-  my $pred_subs      = $opts{predicates} || {};
-  my $construct_subs = $opts{constructors} || [defined($opts{constructor}) ? $opts{constructor} : ()];
-
-  foreach my $subname (keys %$read_subs) {
-    my $hashkey = $read_subs->{$subname};
-    _generate_method($caller_pkg, $subname, $hashkey, $replace, $chained, "getter");
-  }
-
-  foreach my $subname (keys %$set_subs) {
-    my $hashkey = $set_subs->{$subname};
-    _generate_method($caller_pkg, $subname, $hashkey, $replace, $chained, "setter");
-  }
-
-  foreach my $subname (keys %$acc_subs) {
-    my $hashkey = $acc_subs->{$subname};
-    _generate_method($caller_pkg, $subname, $hashkey, $replace, $chained, "accessor");
-  }
-
-  foreach my $subname (keys %$pred_subs) {
-    my $hashkey = $pred_subs->{$subname};
-    _generate_method($caller_pkg, $subname, $hashkey, $replace, $chained, "predicate");
-  }
+  # TODO: Refactor. Move more duplicated code to ::Heavy
   
-  foreach my $subname (@$construct_subs) {
-    _generate_method($caller_pkg, $subname, "", $replace, $chained, "constructor");
+  my $read_subs      = _make_hash($opts{getters} || {});
+  my $set_subs       = _make_hash($opts{setters} || {});
+  my $acc_subs       = _make_hash($opts{accessors} || {});
+  my $pred_subs      = _make_hash($opts{predicates} || {});
+  my $construct_subs = $opts{constructors} || [defined($opts{constructor}) ? $opts{constructor} : ()];
+  my $true_subs      = $opts{true} || [];
+  my $false_subs     = $opts{false} || [];
+
+  foreach my $subtype ( ["getter", $read_subs],
+                        ["setter", $set_subs],
+                        ["accessor", $acc_subs],
+                        ["pred_subs", $pred_subs] )
+  {
+    my $subs = $subtype->[1];
+    foreach my $subname (keys %$subs) {
+      my $hashkey = $subs->{$subname};
+      _generate_method($caller_pkg, $subname, $hashkey, \%opts, $subtype->[0]);
+    }
+  }
+
+  foreach my $subtype ( ["constructor", $construct_subs],
+                        ["true", $true_subs],
+                        ["false", $false_subs] )
+  {
+    foreach my $subname (@{$subtype->[1]}) {
+      _generate_method($caller_pkg, $subname, "", \%opts, $subtype->[0]);
+    }
   }
 }
 
 sub _generate_method {
-  my ($caller_pkg, $subname, $hashkey, $replace, $chained, $type) = @_;
+  my ($caller_pkg, $subname, $hashkey, $opts, $type) = @_;
 
-  if (not defined $hashkey) {
-    croak("Cannot use undef as a hash key for generating an XS $type accessor. (Sub: $subname)");
-  }
+  croak("Cannot use undef as a hash key for generating an XS $type accessor. (Sub: $subname)")
+    if not defined $hashkey;
 
-  if ($subname !~ /::/) {
-    $subname = "${caller_pkg}::$subname";
-  }
+  $subname = "${caller_pkg}::$subname" if $subname !~ /::/;
 
-  if (not $replace) {
-    my $sub_package = $subname;
-    $sub_package =~ s/([^:]+)$// or die;
-    my $bare_subname = $1;
-    
-    my $sym;
-    {
-      no strict 'refs';
-      $sym = \%{"$sub_package"};
-    }
-    no warnings;
-    local *s = $sym->{$bare_subname};
-    my $coderef = *s{CODE};
-    if ($coderef) {
-      croak("Cannot replace existing subroutine '$bare_subname' in package '$sub_package' with XS $type accessor. If you wish to force a replacement, add the 'replace => 1' parameter to the arguments of 'use ".__PACKAGE__."'.");
-    }
-  }
+  Class::XSAccessor::Heavy::check_sub_existence($subname) if not $opts->{replace};
 
   if ($type eq 'getter') {
     newxs_getter($subname, $hashkey);
   }
   elsif ($type eq 'setter') {
-    newxs_setter($subname, $hashkey, $chained);
+    newxs_setter($subname, $hashkey, $opts->{chained}||0);
   }
   elsif ($type eq 'predicate') {
     newxs_predicate($subname, $hashkey);
@@ -93,8 +87,14 @@ sub _generate_method {
   elsif ($type eq 'constructor') {
     newxs_constructor($subname);
   }
+  elsif ($type eq 'true') {
+    newxs_boolean($subname, 1);
+  }
+  elsif ($type eq 'false') {
+    newxs_boolean($subname, 0);
+  }
   else {
-    newxs_accessor($subname, $hashkey, $chained);
+    newxs_accessor($subname, $hashkey, $opts->{chained}||0);
   }
 }
 
@@ -126,7 +126,10 @@ Class::XSAccessor - Generate fast XS accessors without runtime compilation
     predicates => {
       has_foo => 'foo',
       has_bar => 'bar',
-    };
+    }
+    true => [ 'is_token', 'is_whitespace' ],
+    false => [ 'significant' ];
+
   # The imported methods are implemented in fast XS.
   
   # normal class code here.
@@ -144,7 +147,7 @@ Since version 0.10, the module can also generate simple constructors
 (implemented in XS) for you. Simply supply the
 C<constructor =E<gt> 'constructor_name'> option or the
 C<constructors =E<gt> ['new', 'create', 'spawn']> option.
-These constructors do the equivalent of the following perl code:
+These constructors do the equivalent of the following Perl code:
 
   sub new {
     my $class = shift;
@@ -155,7 +158,7 @@ That means they can be called on objects and classes but will not
 clone objects entirely. Parameters to C<new()> are added to the
 object.
 
-The XS accessor methods were between 1.6 and 2.5 times faster than typical
+The XS accessor methods are between 2.6 and 3.4 times faster than typical
 pure-perl accessors in some simple benchmarking.
 The lower factor applies to the potentially slightly obscure
 C<sub set_foo_pp {$_[0]-E<gt>{foo} = $_[1]}>, so if you usually
@@ -170,6 +173,12 @@ By default, the setters return the new value that was set
 and the accessors (mutators) do the same. You can change this behaviour
 with the C<chained> option, see below. The predicates obviously return a boolean.
 
+Since version 1.01, you can generate extremely simply methods which
+simply return true or false (and always do so). If that seems like a
+really superfluous thing to you, then think of a large class hierarchy
+with interfaces such as PPI. This is implemented as the C<true>
+and C<false> options, see synopsis.
+
 =head1 OPTIONS
 
 In addition to specifying the types and names of accessors, you can add options
